/* KinokoIndexedDataSection.cc */
/* Created by Enomoto Sanshiro on 19 November 2000. */
/* Last updated by Enomoto Sanshiro on 19 November 2000. */


#include <iostream>
#include <string>
#include "ParaTokenizer.hh"
#include "KinokoDataSource.hh"
#include "KinokoDataSection.hh"
#include "KinokoIndexedDataSection.hh"

using namespace std;


TKinokoIndexedDataSection::TKinokoIndexedDataSection(TKinokoDataSource* DataSource, const string& SectionName)
: TKinokoDataSection(DataSource, SectionName)
{
    _AddressWidth = 0;
    _DataWidth = 0;
    _ElementDepth = 0;

    _Formatter = 0;
    _Scanner = 0;
}

TKinokoIndexedDataSection::TKinokoIndexedDataSection(TKinokoDataSource* DataSource, const string& SectionName, int SectionId)
: TKinokoDataSection(DataSource, SectionName, SectionId)
{
    _AddressWidth = 0;
    _DataWidth = 0;
    _ElementDepth = 0;

    _Formatter = 0;
    _Scanner = 0;
}

TKinokoIndexedDataSection::~TKinokoIndexedDataSection()
{
    delete _Formatter;
    delete _Scanner;
}

void TKinokoIndexedDataSection::SetWidth(unsigned AddressWidth, unsigned DataWidth)
{
    _AddressWidth = AddressWidth;
    _DataWidth = DataWidth;
}

void TKinokoIndexedDataSection::SetElementDepth(unsigned ElementDepth)
{
    _ElementDepth = ElementDepth;
}

unsigned TKinokoIndexedDataSection::AddressWidth(void) const
{
    return _AddressWidth;
}

unsigned TKinokoIndexedDataSection::DataWidth(void) const
{
    return _DataWidth;
}

unsigned TKinokoIndexedDataSection::ElementDepth(void) const
{
    return _ElementDepth;
}

int TKinokoIndexedDataSection::SectionType(void) const
{
    return TKinokoDataSection::SectionType_Indexed;
}

void TKinokoIndexedDataSection::ReadFrom(TParaTokenizer& Tokenizer) throw(TKinokoException)
{
    unsigned AddressWidth = 0, DataWidth = 0, ElementDepth = 0;

    try {
	TParaToken Token;
	if (Tokenizer.LookAhead().Is("(")) {
	    Tokenizer.Next().MustBe("(");

	    Tokenizer.Next().MustBe("address");
	    Tokenizer.Next().MustBe(":");
	    Tokenizer.Next().MustBe("int");
	    Tokenizer.Next().MustBe("-");
	    AddressWidth = Tokenizer.Next().AsLong();
	    Tokenizer.Next().MustBe("bit");
	    Tokenizer.Next().MustBe(",");
	    Tokenizer.Next().MustBe("data");
	    Tokenizer.Next().MustBe(":");
	    Tokenizer.Next().MustBe("int");
	    Tokenizer.Next().MustBe("-");
	    DataWidth = Tokenizer.Next().AsLong();
	    Tokenizer.Next().MustBe("bit");
	    Tokenizer.Next().MustBe(")");

	    if (AddressWidth + DataWidth > 32) {
		AddressWidth = 32;
		DataWidth = 32;
	    }
	    else {
		AddressWidth = 32 - DataWidth;
	    }
	}

	if (Tokenizer.LookAhead().Is("{")) {
	    //... New address/data width specifier should come here ...//
	    TKinokoDataSource::DiscardUnknownEntry(Tokenizer);
	}
	else {
	    Tokenizer.Next().MustBe(";");
	}
    }
    catch (TScriptException &e) {
	throw TKinokoException(
	    "TKinokoIndexedDataSection::ReadFrom()",
	    "script exception: " + e.Message()
	);
    }

    SetWidth(AddressWidth, DataWidth);
    SetElementDepth(ElementDepth);
}

void TKinokoIndexedDataSection::WriteTo(ostream& os, const string& Indent)
{
    os << Indent << "section \"" << _SectionName << "\"<" << _SectionId << ">";
    os << ": indexed(";
    os << "address: int-" << _AddressWidth << "bit";
    os << ", ";
    os << "data: int-" << _DataWidth << "bit";
#if 0
    os << ", ";
    os << "depth: " << _ElementDepth;
#endif
    os << ");" << endl;
}



TKinokoIndexedDataSectionFormatter::TKinokoIndexedDataSectionFormatter(TKinokoIndexedDataSection* DataSection)
: TKinokoDataSectionFormatter(DataSection)
{
    _AddressWidth = DataSection->AddressWidth();
    _DataWidth = DataSection->DataWidth();

    _UsesTwoWords = (_AddressWidth + _DataWidth > 32);
}

TKinokoIndexedDataSectionFormatter::~TKinokoIndexedDataSectionFormatter()
{
}



TKinokoIndexedDataSectionScanner::TKinokoIndexedDataSectionScanner(TKinokoIndexedDataSection* DataSection)
{
    _AddressWidth = DataSection->AddressWidth();
    _DataWidth = DataSection->DataWidth();
    _ElementWidth = _AddressWidth + _DataWidth;
}

TKinokoIndexedDataSectionScanner::~TKinokoIndexedDataSectionScanner()
{
}
