/* KinokoCanvasDataScale.hh */
/* Created by Enomoto Sanshiro on 11 July 2000. */
/* Last updated by Enomoto Sanshiro on 7 December 2003. */


#ifndef __KinokoCanvasDataScale_hh
#define __KinokoCanvasDataScale_hh


#include <vector>


class TKinokoCanvasDataScale {
  public:
    TKinokoCanvasDataScale(float Min = 0, float Max = 1.0, int MaxNumberOfDivisions = 10);
    virtual ~TKinokoCanvasDataScale();
    virtual TKinokoCanvasDataScale& Rescale(float Min, float Max, int MaxNumberOfDivisions = 10);
    virtual float Min(void) const;
    virtual float Max(void) const;
    virtual float Width(void) const;
    virtual int NumberOfDivisions(void);
    virtual float DivisionValueOf(int DivisionIndex);
    virtual int NumberOfTicks(void);
    virtual float TickValueOf(int TickIndex);
  protected:
    static std::pair<float, float> DecideSteps(float Min, float Max, int MaxNumberOfSteps);
  protected:
    float _Min, _Max, _Width;
    int _MaxNumberOfDivisions;
    float _FirstDivision, _LastDivision, _DivisionStep;
    int _NumberOfDivisions;
    float _FirstTick, _TickStep;
    int _NumberOfTicks;
  private:
    float _UsedMin, _UsedMax;
};


class TKinokoCanvasLogDataScale: public TKinokoCanvasDataScale {
  public:
    TKinokoCanvasLogDataScale(float Min = 0.1, float Max = 1.0, int MaxNumberOfDivisions = 10);
    virtual ~TKinokoCanvasLogDataScale();
    virtual TKinokoCanvasDataScale& Rescale(float Min, float Max, int MaxNumberOfDivisions = 10);
    virtual int NumberOfDivisions(void);
    virtual float DivisionValueOf(int DivisionIndex);
    virtual int NumberOfTicks(void);
    virtual float TickValueOf(int TickIndex);
  protected:
    bool _IsUsingNormalScale;
    int _TickIndexOffset;
};


#endif
