/* KinokoCanvasObject.cc */
/* Created by Enomoto Sanshiro on 10 July 2000. */
/* Last updated by Enomoto Sanshiro on 8 April 2001. */


#include <iostream>
#include <string>
#include "KinokoCanvas.hh"
#include "KinokoCanvasImageArea.hh"
#include "KinokoCanvasTypesetter.hh"
#include "KinokoCanvasObject.hh"

using namespace std;



TKinokoCanvasObject::TKinokoCanvasObject(TKinokoCanvas* Canvas, TKinokoCanvasImageArea* ImageArea)
{
    _Canvas = Canvas;
    _ImageArea = ImageArea;

    _OffsetX = 0;
    _OffsetY = 0;
    _Width = 0;
    _Height = 0;

    for (int i = 0; i < _NumberOfEnumFonts; i++) {
	_MathTextComposerList[i] = 0;
    }
    _Typesetter = 0;
}

TKinokoCanvasObject::~TKinokoCanvasObject()
{
    for (int i = 0; i < _NumberOfEnumFonts; i++) {
	delete _MathTextComposerList[i];
    }

    delete _Typesetter;
}

void TKinokoCanvasObject::Initialize(void)
{
    TKinokoShellObject::Initialize();

    _ColorList[Color_Foreground] = "black";
    _ColorList[Color_Background] = "white";

    _FontList[Font_Normal] = make_pair(string("helvetica"), 14);
    _FontList[Font_Title] = make_pair(string("helvetica"), 20);
#if 0
    _FontList[Font_Label] = make_pair(string("helvetica"), 16);
#else
    _FontList[Font_Label] = make_pair(string("helvetica-bold"), 16);
#endif

    for (unsigned i = 0; i < _NumberOfEnumColors; i++) {
	_ColorIndexList[i] = _ImageArea->AllocateColor(_ColorList[i]);
    }
    _ImageArea->SetColor(_ColorIndexList[Color_Foreground]);

    for (unsigned i = 0; i < _NumberOfEnumFonts; i++) {
	_FontIndexList[i] = _ImageArea->LoadFont(
	    _FontList[i].first, _FontList[i].second
	);
	_MathTextComposerList[i] = new TKinokoCanvasMathTextComposer(
	    _ImageArea
	);
	_MathTextComposerList[i]->SetFont(
	    _FontList[i].first, _FontList[i].second
	);
    }

    _Typesetter = new TKinokoCanvasTypesetter(_ImageArea);

    _ImageArea->SetFont(_FontIndexList[Font_Label]);
    _LabelCharacterHeight = _ImageArea->TextHeightOf("0");
    _LabelCharacterWidth = _ImageArea->TextWidthOf("0");

    _ImageArea->SetFont(_FontIndexList[Font_Normal]);
    _NormalFontLineHeight = _ImageArea->TextHeightOf("Ap");
}

int TKinokoCanvasObject::Construct(istream& InputStream)
{
    int Result = 1;

    float OffsetX, OffsetY, Width, Height;
    if (InputStream >> OffsetX >> OffsetY >> Width >> Height) {
	OffsetX = (OffsetX / 100) * _ImageArea->Width();
	OffsetY = (OffsetY / 100) * _ImageArea->Height();
	Width = (Width / 100) * _ImageArea->Width();
	Height = (Height / 100) * _ImageArea->Height();
	
	SetPosition((int) OffsetX, (int) OffsetY, (int) Width, (int) Height);
	_Canvas->AddCanvasObject(this);
    }
    else {
	Result = 0;
    }

    return Result;
}

void TKinokoCanvasObject::SetPosition(int OffsetX, int OffsetY, int Width, int Height)
{
    _OffsetX = OffsetX;
    _OffsetY = OffsetY;
    _Width = Width;
    _Height = Height;
}

bool TKinokoCanvasObject::Includes(int X, int Y)
{
    return (
	(X > _OffsetX) && (Y > _OffsetY) && 
	(X < _OffsetX + _Width) && (Y < _OffsetY + _Height)
    );
}

vector<string> TKinokoCanvasObject::ActionList(void)
{
    return vector<string>();
}
    
int TKinokoCanvasObject::ProcessAction(const string& ActionName)
{
    return 0;
}

void TKinokoCanvasObject::Clear(void)
{
    int OldColor = _ImageArea->SetColor(_ColorIndexList[Color_Background]);
    _ImageArea->DrawRectFill(
	_OffsetX, _OffsetY, _OffsetX + _Width, _OffsetY + _Height
    );
    _ImageArea->SetColor(OldColor);
}
